package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.JwtService;
import cz.muni.fi.pa165.token.TokenRepository;
import cz.muni.fi.pa165.user.Role;
import cz.muni.fi.pa165.user.UserDo;
import cz.muni.fi.pa165.user.UserRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@SpringBootTest(classes = CoreApplication.class)
@AutoConfigureMockMvc
@ExtendWith({SpringExtension.class, MockitoExtension.class})
public class AuthenticationServiceTest {

    @Mock
    private UserRepository userRepository;

    @Mock
    private PasswordEncoder passwordEncoder;

    @Mock
    private JwtService jwtService;

    @Mock
    private AuthenticationManager authenticationManager;

    @Mock
    private TokenRepository tokenRepository;

    @InjectMocks
    private AuthenticationService authenticationService;

    @BeforeEach
    public void setUp() {
        authenticationService = new AuthenticationService(
                userRepository, tokenRepository, passwordEncoder, jwtService, authenticationManager);
    }

    @Test
    void registerTest() {
        RegisterRequest registerRequest = createRegisterRequest();
        UserDo user = new UserDo(
                registerRequest.getFirstname(),
                registerRequest.getLastname(),
                registerRequest.getUsername(),
                passwordEncoder.encode(registerRequest.getPassword()),
                Role.USER);

        String token = "access_token";
        String refreshToken = "refresh_token";
        when(userRepository.save(any(UserDo.class))).thenReturn(user);
        when(jwtService.generateToken(Mockito.any(UserDo.class))).thenReturn(token);
        when(jwtService.generateRefreshToken(Mockito.any(UserDo.class))).thenReturn(refreshToken);

        AuthenticationResponse response = authenticationService.register(registerRequest);

        assertThat(response).isNotNull();
        assertThat(response.getAccessToken()).isNotNull();
        assertThat(response.getRefreshToken()).isNotNull();
        assertThat(response.getAccessToken()).isEqualTo(token);
        assertThat(response.getRefreshToken()).isEqualTo(refreshToken);
        Mockito.verify(userRepository, Mockito.times(1)).save(Mockito.any(UserDo.class));
    }

    @Test
    void authenticateTest() {
        AuthenticationRequest authenticationRequest = createAuthenticationRequest();
        UserDo user = new UserDo(
                "Peter",
                "Palicka",
                authenticationRequest.getUsername(),
                passwordEncoder.encode(authenticationRequest.getPassword()),
                Role.USER);
        String token = "access_token";
        String refreshToken = "refresh_token";
        when(userRepository.findByUsername(authenticationRequest.getUsername())).thenReturn(user);
        when(jwtService.generateToken(Mockito.any(UserDo.class))).thenReturn(token);
        when(jwtService.generateRefreshToken(Mockito.any(UserDo.class))).thenReturn(refreshToken);

        AuthenticationResponse response = authenticationService.authenticate(authenticationRequest);

        assertThat(response).isNotNull();
        assertThat(response.getAccessToken()).isNotNull();
        assertThat(response.getRefreshToken()).isNotNull();
        assertThat(response.getAccessToken()).isEqualTo(token);
        assertThat(response.getRefreshToken()).isEqualTo(refreshToken);
        Mockito.verify(userRepository, Mockito.times(1)).findByUsername(authenticationRequest.getUsername());
    }

    private RegisterRequest createRegisterRequest() {
        return new RegisterRequest("Peter", "Palicka", "peterpalicka", "password123456");
    }

    private AuthenticationRequest createAuthenticationRequest() {
        return new AuthenticationRequest("peterpalicka", "password123456");
    }
}
